@@ -3,19 +3,21 @@ Gym environment for simulation with EnergyPlus.
 """
 
 import os
+import random
+from sqlite3 import DatabaseError
 from typing import Any, Dict, List, Optional, Tuple, Union
 
-import gym
+import gymnasium as gym
 import numpy as np
 
 from sinergym.simulators import EnergyPlus
-from sinergym.utils.constants import PKG_DATA_PATH
+from sinergym.utils.common import export_actuators_to_excel
 from sinergym.utils.rewards import ExpReward, LinearReward
 
 
 class EplusEnv(gym.Env):
 
-    metadata = {'render.modes': ['human']}
+    metadata = {'render_modes': ['human']}
 
     # ---------------------------------------------------------------------------- #
     #                            ENVIRONMENT CONSTRUCTOR                           #
@@ -23,12 +25,12 @@ class EplusEnv(gym.Env):
     def __init__(
         self,
         idf_file: str,
-        weather_file: str,
+        weather_file: Union[str, List[str]],
         observation_space: gym.spaces.Box = gym.spaces.Box(
-            low=-5e6, high=5e6, shape=(4,)),
+            low=-5e6, high=5e6, shape=(4,), dtype=np.float32),
         observation_variables: List[str] = [],
         action_space: Union[gym.spaces.Box, gym.spaces.Discrete] = gym.spaces.Box(
-            low=0, high=0, shape=(0,)),
+            low=0, high=0, shape=(0,), dtype=np.float32),
         action_variables: List[str] = [],
         action_mapping: Dict[int, Tuple[float, ...]] = {},
         weather_variability: Optional[Tuple[float]] = None,
@@ -36,6 +38,7 @@ class EplusEnv(gym.Env):
         reward_kwargs: Optional[Dict[str, Any]] = {},
         act_repeat: int = 1,
         max_ep_data_store_num: int = 10,
+        action_definition: Optional[Dict[str, Any]] = None,
         env_name: str = 'eplus-env-v1',
         config_params: Optional[Dict[str, Any]] = None
     ):
@@ -43,7 +46,7 @@ class EplusEnv(gym.Env):
 
         Args:
             idf_file (str): Name of the IDF file with the building definition.
-            weather_file (str): Name of the EPW file for weather conditions.
+            weather_file (Union[str,List[str]]): Name of the EPW file for weather conditions. It can be specified a list of weathers files in order to sample a weather in each episode randomly.
             observation_space (gym.spaces.Box, optional): Gym Observation Space definition. Defaults to an empty observation_space (no control).
             observation_variables (List[str], optional): List with variables names in IDF. Defaults to an empty observation variables (no control).
             action_space (Union[gym.spaces.Box, gym.spaces.Discrete], optional): Gym Action Space definition. Defaults to an empty action_space (no control).
@@ -54,6 +57,7 @@ class EplusEnv(gym.Env):
             reward_kwargs (Optional[Dict[str, Any]], optional): Parameters to be passed to the reward function. Defaults to empty dict.
             act_repeat (int, optional): Number of timesteps that an action is repeated in the simulator, regardless of the actions it receives during that repetition interval.
             max_ep_data_store_num (int, optional): Number of last sub-folders (one for each episode) generated during execution on the simulation.
+            action_definition (Optional[Dict[str, Any]): Dict with building components to being controlled by Sinergym automatically if it is supported. Default value to None.
             env_name (str, optional): Env name used for working directory generation. Defaults to eplus-env-v1.
             config_params (Optional[Dict[str, Any]], optional): Dictionary with all extra configuration for simulator. Defaults to None.
         """
@@ -63,11 +67,14 @@ class EplusEnv(gym.Env):
         # ---------------------------------------------------------------------------- #
         eplus_path = os.environ['EPLUS_PATH']
         bcvtb_path = os.environ['BCVTB_PATH']
-        self.pkg_data_path = PKG_DATA_PATH
 
-        self.idf_path = os.path.join(self.pkg_data_path, 'buildings', idf_file)
-        self.weather_path = os.path.join(
-            self.pkg_data_path, 'weather', weather_file)
+        # IDF file
+        self.idf_file = idf_file
+        # EPW file(s) (str or List of EPW's)
+        if isinstance(weather_file, str):
+            self.weather_files = [weather_file]
+        else:
+            self.weather_files = weather_file
 
         # ---------------------------------------------------------------------------- #
         #                             Variables definition                             #
@@ -76,6 +83,8 @@ class EplusEnv(gym.Env):
         self.variables['observation'] = observation_variables
         self.variables['action'] = action_variables
 
+        self.name = env_name
+
         # ---------------------------------------------------------------------------- #
         #                                   Simulator                                  #
         # ---------------------------------------------------------------------------- #
@@ -83,14 +92,20 @@ class EplusEnv(gym.Env):
             env_name=env_name,
             eplus_path=eplus_path,
             bcvtb_path=bcvtb_path,
-            idf_path=self.idf_path,
-            weather_path=self.weather_path,
+            idf_file=self.idf_file,
+            weather_files=self.weather_files,
             variables=self.variables,
             act_repeat=act_repeat,
             max_ep_data_store_num=max_ep_data_store_num,
+            action_definition=action_definition,
             config_params=config_params
         )
 
+        # ---------------------------------------------------------------------------- #
+        #                       Detection of controllable planners                     #
+        # ---------------------------------------------------------------------------- #
+        self.schedulers = self.get_schedulers()
+
         # ---------------------------------------------------------------------------- #
         #        Adding simulation date to observation (not needed in simulator)       #
         # ---------------------------------------------------------------------------- #
@@ -99,14 +114,18 @@ class EplusEnv(gym.Env):
                                          'day', 'hour'] + self.variables['observation']
 
         # ---------------------------------------------------------------------------- #
-        #                              Weather variability                             #
+        #                          reset default options                               #
         # ---------------------------------------------------------------------------- #
-        self.weather_variability = weather_variability
+        self.default_options = {}
+        # Weather Variability
+        if weather_variability:
+            self.default_options['weather_variability'] = weather_variability
+        # ... more reset option implementations here
 
         # ---------------------------------------------------------------------------- #
         #                               Observation Space                              #
         # ---------------------------------------------------------------------------- #
-        self.observation_space = observation_space
+        self._observation_space = observation_space
 
         # ---------------------------------------------------------------------------- #
         #                                 Action Space                                 #
@@ -120,16 +139,18 @@ class EplusEnv(gym.Env):
         # Discrete
         if self.flag_discrete:
             self.action_mapping = action_mapping
-            self.action_space = action_space
+            self._action_space = action_space
         # Continuous
         else:
             # Defining action values setpoints (one per value)
             self.setpoints_space = action_space
 
-            self.action_space = gym.spaces.Box(
+            self._action_space = gym.spaces.Box(
                 # continuous_action_def[2] --> shape
-                low=np.repeat(-1, action_space.shape[0]),
-                high=np.repeat(1, action_space.shape[0]),
+                low=np.array(
+                    np.repeat(-1, action_space.shape[0]), dtype=np.float32),
+                high=np.array(
+                    np.repeat(1, action_space.shape[0]), dtype=np.float32),
                 dtype=action_space.dtype
             )
 
@@ -148,16 +169,34 @@ class EplusEnv(gym.Env):
     # ---------------------------------------------------------------------------- #
     #                                     RESET                                    #
     # ---------------------------------------------------------------------------- #
-    def reset(self) -> np.ndarray:
+    def reset(self,
+              seed: Optional[int] = None,
+              options: Optional[Dict[str,
+                                     Any]] = None) -> Tuple[np.ndarray,
+                                                            Dict[str,
+                                                                 Any]]:
         """Reset the environment.
 
-        Returns:
-            np.ndarray: Current observation.
-        """
-        # Change to next episode
-        _, obs, _ = self.simulator.reset(self.weather_variability)
+        Args:
+            seed (Optional[int]): The seed that is used to initialize the environment's episode (np_random). if value is None, a seed will be chosen from some source of entropy. Defaults to None.
+            options (Optional[Dict[str, Any]]):Additional information to specify how the environment is reset. Defaults to None.
 
-        return np.array(obs, dtype=np.float32)
+        Returns:
+            Tuple[np.ndarray,Dict[str,Any]]: Current observation and info context with additional information.
+        """
+        # We need the following line to seed self.np_random
+        super().reset(seed=seed)
+
+        # Change to next episode
+        # if no options specified and environment has default reset options
+        if not options and len(self.default_options) > 0:
+            obs, info = self.simulator.reset(
+                self.default_options)
+        else:
+            obs, info = self.simulator.reset(
+                options)
+
+        return np.array(obs, dtype=np.float32), info
 
     # ---------------------------------------------------------------------------- #
     #                                     STEP                                     #
@@ -169,45 +208,34 @@ class EplusEnv(gym.Env):
                            np.ndarray,
                            List[Any],
                            Tuple[Any]]
-             ) -> Tuple[np.ndarray, float, bool, Dict[str, Any]]:
+             ) -> Tuple[np.ndarray, float, bool, bool, Dict[str, Any]]:
         """Sends action to the environment
 
         Args:
             action (Union[int, float, np.integer, np.ndarray, List[Any], Tuple[Any]]): Action selected by the agent.
 
         Returns:
-            Tuple[np.ndarray, float, bool, Dict[str, Any]]: Observation for next timestep, reward obtained, Whether the episode has ended or not and a dictionary with extra information
+            Tuple[np.ndarray, float, bool, Dict[str, Any]]: Observation for next timestep, reward obtained, Whether the episode has ended or not, Whether episode has been truncated or not, and a dictionary with extra information
         """
 
         # Get action
         action_ = self._get_action(action)
         # Send action to the simulator
         self.simulator.logger_main.debug(action_)
-        # time_info = (current simulation year, month, day, hour, time_elapsed)
-        time_elapsed, obs, done = self.simulator.step(action_)
+        # Execute action in simulation
+        obs, terminated, truncated, info = self.simulator.step(action_)
         # Create dictionary with observation
         self.obs_dict = dict(zip(self.variables['observation'], obs))
 
         # Calculate reward
         reward, terms = self.reward_fn()
 
-        # Extra info
-        info = {
-            'timestep': int(
-                time_elapsed / self.simulator._eplus_run_stepsize),
-            'time_elapsed': int(time_elapsed),
-            'year': self.obs_dict['year'],
-            'month': self.obs_dict['month'],
-            'day': self.obs_dict['day'],
-            'hour': self.obs_dict['hour'],
-            'total_power': terms.get('total_energy'),
-            'total_power_no_units': terms.get('reward_energy'),
-            'comfort_penalty': terms.get('reward_comfort'),
-            'temperatures': terms.get('temperatures'),
-            'out_temperature': self.obs_dict['Site Outdoor Air Drybulb Temperature(Environment)'],
-            'action_': action_}
+        # info update with reward information
+        info.update({'reward': reward})
+        info.update(terms)
 
-        return np.array(obs, dtype=np.float32), reward, done, info
+        return np.array(
+            obs, dtype=np.float32), reward, terminated, truncated, info
 
     # ---------------------------------------------------------------------------- #
     #                                RENDER (empty)                                #
@@ -231,7 +259,34 @@ class EplusEnv(gym.Env):
     # ---------------------------------------------------------------------------- #
     #                           Environment functionality                          #
     # ---------------------------------------------------------------------------- #
-    def _get_action(self, action: Any):
+    def get_schedulers(self, path: Optional[str] = None) -> Dict[str, Any]:
+        """Extract all schedulers available in the building model to be controlled.
+
+        Args:
+            path (str, optional): If path is specified, then this method export a xlsx file version in addition to return the dictionary.
+
+        Returns:
+            Dict[str, Any]: Python Dictionary: For each scheduler found, it shows type value and where this scheduler is present (Object name, Object field and Object type).
+        """
+        schedulers = self.simulator._config.schedulers
+        if path is not None:
+            export_actuators_to_excel(actuators=schedulers, path=path)
+        return schedulers
+
+    def get_zones(self) -> List[str]:
+        """Get the zone names available in the building model of that environment.
+
+        Returns:
+            List[str]: List of the zone names.
+        """
+        return self.simulator._config.idf_zone_names
+
+    def _get_action(self, action: Any) -> Union[int,
+                                                float,
+                                                np.integer,
+                                                np.ndarray,
+                                                List[Any],
+                                                Tuple[Any]]:
         """Transform the action for sending it to the simulator."""
 
         # Get action depending on flag_discrete
@@ -246,13 +301,14 @@ class EplusEnv(gym.Env):
             elif isinstance(action, tuple) or isinstance(action, list):
                 # stable-baselines DQN bug prevention
                 if len(action) == 1:
-                    setpoints = self.action_mapping[action.item()]
+                    setpoints = self.action_mapping[action[0]]
                 else:
                     setpoints = action
             elif isinstance(action, np.ndarray):
                 setpoints = self.action_mapping[action.item()]
             else:
-                print("ERROR: ", type(action))
+                raise RuntimeError(
+                    'action type not supported by Sinergym environment')
             action_ = list(setpoints)
         else:
             # transform action to setpoints simulation
@@ -283,14 +339,19 @@ class EplusEnv(gym.Env):
         action_ = []
 
         for i, value in enumerate(action):
-            if self.action_space.low[i] <= value <= self.action_space.high[i]:
-                a_max_min = self.action_space.high[i] - \
-                    self.action_space.low[i]
+            if self._action_space.low[i] <= value <= self._action_space.high[i]:
+                a_max_min = self._action_space.high[i] - \
+                    self._action_space.low[i]
                 sp_max_min = self.setpoints_space.high[i] - \
                     self.setpoints_space.low[i]
 
                 action_.append(
-                    self.setpoints_space.low[i] + (value - self.action_space.low[i]) * sp_max_min / a_max_min)
+                    self.setpoints_space.low[i] +
+                    (
+                        value -
+                        self._action_space.low[i]) *
+                    sp_max_min /
+                    a_max_min)
             else:
                 # If action is outer action_space already, it don't need
                 # transformation
@@ -302,7 +363,7 @@ class EplusEnv(gym.Env):
         """This method checks that environment definition is correct and it has not inconsistencies.
         """
         # OBSERVATION
-        assert len(self.variables['observation']) == self.observation_space.shape[
+        assert len(self.variables['observation']) == self._observation_space.shape[
             0], 'Observation space has not the same length than variable names specified.'
 
         # ACTION
@@ -311,17 +372,40 @@ class EplusEnv(gym.Env):
                 self, 'action_mapping'), 'Discrete environment: action mapping should have been defined.'
             assert not hasattr(
                 self, 'setpoints_space'), 'Discrete environment: setpoints space should not have been defined.'
-            assert self.action_space.n == len(
+            assert self._action_space.n == len(
                 self.action_mapping), 'Discrete environment: The length of the action_mapping must match the dimension of the discrete action space.'
             for values in self.action_mapping.values():
                 assert len(values) == len(
                     self.variables['action']), 'Discrete environment: Action mapping tuples values must have the same length than action variables specified.'
         else:
-            assert len(
-                self.variables['action']) == self.action_space.shape[0], 'Action space shape must match with number of action variables specified.'
+            assert len(self.variables['action']) == self._action_space.shape[
+                0], 'Action space shape must match with number of action variables specified.'
             assert hasattr(
                 self, 'setpoints_space'), 'Continuous environment: setpoints_space attribute should have been defined.'
             assert not hasattr(
                 self, 'action_mapping'), 'Continuous environment: action mapping should not have been defined.'
-            assert len(self.action_space.low) == len(self.variables['action']) and len(self.action_space.high) == len(
+            assert len(self._action_space.low) == len(self.variables['action']) and len(self._action_space.high) == len(
                 self.variables['action']), 'Continuous environment: low and high values action space definition should have the same number of values than action variables.'
+
+    # ---------------------------------------------------------------------------- #
+    #                                  Properties                                  #
+    # ---------------------------------------------------------------------------- #
+    @property
+    def action_space(
+        self,
+    ) -> gym.spaces.Space[Any] | gym.spaces.Space[Any]:
+        return self._action_space
+
+    @action_space.setter
+    def action_space(self, space: gym.spaces.Space[Any]):
+        self._action_space = space
+
+    @property
+    def observation_space(
+        self,
+    ) -> gym.spaces.Space[Any] | gym.spaces.Space[Any]:
+        return self._observation_space
+
+    @observation_space.setter
+    def observation_space(self, space: gym.spaces.Space[Any]):
+        self._observation_space = space
