@@ -33,21 +33,22 @@ class EnergyPlus(object):
     def __init__(
             self,
             eplus_path: str,
-            weather_path: str,
             bcvtb_path: str,
-            idf_path: str,
+            weather_files: List[str],
+            idf_file: str,
             env_name: str,
             variables: Dict[str, List[str]],
             act_repeat: int = 1,
             max_ep_data_store_num: int = 10,
+            action_definition: Optional[Dict[str, Any]] = None,
             config_params: Optional[Dict[str, Any]] = None):
         """EnergyPlus simulation class.
 
         Args:
             eplus_path (str):  EnergyPlus installation path.
-            weather_path (str): EnergyPlus weather file (.epw) path.
             bcvtb_path (str): BCVTB installation path.
-            idf_path (str): EnergyPlus input description file (.idf) path.
+            weather_files (List[str]): EnergyPlus weather file list (sampling one in each episode).
+            idf_file (str): EnergyPlus input description file (.idf) file.
             env_name (str): The environment name.
             variables (Dict[str,List[str]]): Variables list with observation and action keys in a dictionary.
             act_repeat (int, optional): The number of times to repeat the control action. Defaults to 1.
@@ -55,7 +56,7 @@ class EnergyPlus(object):
             config_params (Optional[Dict[str, Any]], optional): Dictionary with all extra configuration for simulator. Defaults to None.
         """
         self._env_name = env_name
-        self._thread_name = threading.current_thread().getName()
+        self._thread_name = threading.current_thread().name
         self.logger_main = Logger().getLogger(
             'EPLUS_ENV_%s_%s_ROOT' %
             (env_name, self._thread_name), LOG_LEVEL_MAIN, LOG_FMT)
@@ -80,8 +81,8 @@ class EnergyPlus(object):
 
         # Path attributes
         self._eplus_path = eplus_path
-        self._weather_path = weather_path
-        self._idf_path = idf_path
+        self._weather_files = weather_files
+        self._idf_file = idf_file
         # Episode existed
         self._episode_existed = False
 
@@ -92,15 +93,20 @@ class EnergyPlus(object):
 
         # Creating models config (with extra params if exits)
         self._config = Config(
-            idf_path=self._idf_path,
-            weather_path=self._weather_path,
+            idf_file=self._idf_file,
+            weather_files=self._weather_files,
             variables=variables,
             env_name=self._env_name,
             max_ep_store=self._max_ep_data_store_num,
+            action_definition=action_definition,
             extra_config=config_params)
 
         # Annotate experiment path in simulator
         self._env_working_dir_parent = self._config.experiment_path
+        # Setting an external interface if IDF building has not got.
+        self.logger_main.info(
+            'Updating idf ExternalInterface object if it is not present...')
+        self._config.set_external_interface()
         # Updating IDF file (Location and DesignDays) with EPW file
         self.logger_main.info(
             'Updating idf Site:Location and SizingPeriod:DesignDay(s) to weather and ddy file...')
@@ -114,6 +120,11 @@ class EnergyPlus(object):
         self.logger_main.info(
             'Setting up extra configuration in building model if exists...')
         self._config.apply_extra_conf()
+        # Setting up action definition automatic manipulation if exists
+        self.logger_main.info(
+            'Setting up action definition in building model if exists...')
+        self._config.adapt_idf_to_action_definition()
+
         # In this lines Epm model is modified but no IDF is stored anywhere yet
 
         # Eplus run info
@@ -131,9 +142,11 @@ class EnergyPlus(object):
         # Stepsize in seconds
         self._eplus_run_stepsize = 3600 / self._eplus_n_steps_per_hour
 
-    def reset(
-        self, weather_variability: Optional[Tuple[float, float, float]] = None
-    ) -> Tuple[float, List[float], bool]:
+    def reset(self,
+              options: Optional[Dict[str,
+                                     Any]] = None) -> Tuple[List[float],
+                                                            Dict[str,
+                                                                 Any]]:
         """Resets the environment.
         This method does the following:
         1. Makes a new EnergyPlus working directory.
@@ -145,19 +158,28 @@ class EnergyPlus(object):
         7. Uses a new weather file if passed.
 
         Args:
-            weather_variability (Optional[Tuple[float, float, float]], optional): Tuple with the sigma, mean and tau for OU process. Defaults to None.
+            options (Optional[Dict[str, Any]]):Additional information to specify how the environment is reset (weather variability for example). Defaults to None.
 
         Returns:
-            Tuple[float, List[float], bool]: The first element is a value with simulation time elapsed;
-            the second element consist on EnergyPlus results in a 1-D list corresponding to the variables in
-            variables.cfg and year, month, day and hour in simulation. The last element is a boolean indicating whether the episode terminates.
+            Tuple[List[float], Dict[str, Any]]: EnergyPlus results in a 1-D list corresponding to the variables in
+            variables.cfg and year, month, day and hour in simulation. A Python dictionary with additional information is included.
         """
+        # options empty
+        if options is None:
+            options = {}
         # End the last episode if exists
         if self._episode_existed:
             self._end_episode()
             self.logger_main.info(
                 'EnergyPlus episode completed successfully. ')
             self._epi_num += 1
+            # Update weather file used if it is specified more than one
+            if len(self._weather_files) > 1:
+                self.logger_main.info(
+                    'Updating weather file for new episode (sampling)...')
+                self._config.update_weather_path()
+                self.logger_main.info('Adapting idf to new weather file...')
+                self._config.adapt_idf_to_epw()
 
         # Create EnergyPlus simulation process
         self.logger_main.info('Creating new EnergyPlus simulation episode...')
@@ -168,7 +190,7 @@ class EnergyPlus(object):
         eplus_working_var_path = self._config.save_variables_cfg()
         eplus_working_out_path = (eplus_working_dir + '/' + 'output')
         eplus_working_weather_path = self._config.apply_weather_variability(
-            variation=weather_variability)
+            variation=options.get('weather_variability'))
 
         self._create_socket_cfg(self._host,
                                 self._port,
@@ -220,21 +242,35 @@ class EnergyPlus(object):
         self._curSimTim = curSimTim
         # Check if episode terminates
         is_terminal = False
-        if curSimTim >= self._eplus_one_epi_len:
+        if curSimTim >= self._eplus_one_epi_len:  # pragma: no cover
             is_terminal = True
         # Change some attributes
         self._conn = conn
         self._eplus_working_dir = eplus_working_dir
         self._episode_existed = True
         # Check termination
-        if is_terminal:
+        if is_terminal:  # pragma: no cover
             self._end_episode()
 
-        return (curSimTim, Dblist, is_terminal)
+        # Setting up info return (additional reset data)
+        info = {
+            'eplus_working_dir': eplus_working_dir,
+            'episode_num': self._epi_num,
+            'socket_host': addr[0],
+            'socket_port': addr[1],
+            'init_year': time_info[0],
+            'init_month': time_info[1],
+            'init_day': time_info[2],
+            'init_hour': time_info[3],
+            'timestep': 0,
+            'time_elapsed': 0
+        }
+
+        return Dblist, info
 
     def step(self, action: Union[int, float, np.integer, np.ndarray, List[Any],
                                  Tuple[Any]]
-             ) -> Tuple[float, List[float], bool]:
+             ) -> Tuple[List[float], bool, bool, Dict[str, Any]]:
         """Executes a given action.
         This method does the following:
         1. Sends a list of floats to EnergyPlus.
@@ -247,9 +283,9 @@ class EnergyPlus(object):
             RuntimeError: When you try to step in an terminated episode (you should be reset before).
 
         Returns:
-            Tuple[float, List[float], bool]: The first element is a float with simulation time elapsed;
-            the second element consist on EnergyPlus results in a 1-D list corresponding to the variables in
-            variables.cfg. The last element is a boolean indicating whether the episode terminates.
+            Tuple[List[float], bool, bool, Dict[str, Any]]: EnergyPlus results in a 1-D list corresponding to the
+            variables in variables.cfg. Second element is whether episode has terminated state, third whether episode
+            has truncated state (always False), and last element is a Python Dictionary with additional information about step
         """
         # Check if terminal
         if self._curSimTim >= self._eplus_one_epi_len:
@@ -290,7 +326,19 @@ class EnergyPlus(object):
         self._curSimTim = curSimTim
         self._last_action = action
 
-        return (curSimTim, Dblist, is_terminal)
+        # info dictionary
+        info = {
+            'timestep': int(
+                curSimTim / self._eplus_run_stepsize),
+            'time_elapsed': int(curSimTim),
+            'year': time_info[0],
+            'month': time_info[1],
+            'day': time_info[2],
+            'hour': time_info[3],
+            'action': action
+        }
+
+        return Dblist, is_terminal, False, info
 
     def _create_eplus(
             self,
@@ -356,7 +404,7 @@ class EnergyPlus(object):
         path_list = file_path.split('/')
         return path_list[-1]
 
-    def _log_subprocess_info(self, out: Any, logger: logging.Logger) -> None:
+    def _log_subprocess_info(self, out: Any, logger: logging.Logger) -> None:  # pragma: no cover
         """Logger info message from subprocess
 
         Args:
@@ -366,7 +414,7 @@ class EnergyPlus(object):
         for line in iter(out.readline, b''):
             logger.info(line.decode())
 
-    def _log_subprocess_err(self, out: Any, logger: logging.Logger) -> None:
+    def _log_subprocess_err(self, out: Any, logger: logging.Logger) -> None:  # pragma: no cover
         """Logger err message from subprocess
 
         Args:
@@ -447,7 +495,7 @@ class EnergyPlus(object):
             os.killpg(self._eplus_process.pid, signal.SIGTERM)
             self._episode_existed = False
 
-    def _run_eplus_outputProcessing(self) -> None:
+    def _run_eplus_outputProcessing(self) -> None:  # pragma: no cover
         # If simulator has not been running with reset at least one time this
         # method is null.
         if hasattr(self, '_eplus_working_dir'):
@@ -465,12 +513,12 @@ class EnergyPlus(object):
     def _assembleMsg(
             self,
             version: int,
-            flag: int,
+            flag: float,
             nDb: int,
             nIn: int,
             nBl: int,
             curSimTim: float,
-            Dblist: Union[float, np.ndarray]) -> str:
+            Dblist: Union[float, np.ndarray, List[float]]) -> str:
         """Assembles the sent message to EnergyPlus based on the protocol.
         The message must be a blank space separated string set with the fields defined as arguments.
 
@@ -538,7 +586,7 @@ class EnergyPlus(object):
         return (version, flag, nDb, nIn, nBl, curSimTim, Dblist)
 
     @property
-    def start_year(self) -> int:
+    def start_year(self) -> int:  # pragma: no cover
         """Returns the EnergyPlus simulation year.
 
         Returns:
@@ -548,7 +596,7 @@ class EnergyPlus(object):
         return self._config.start_year
 
     @property
-    def start_mon(self) -> int:
+    def start_mon(self) -> int:  # pragma: no cover
         """Returns the EnergyPlus simulation start month.
 
         Returns:
@@ -557,7 +605,7 @@ class EnergyPlus(object):
         return self._eplus_run_st_mon
 
     @property
-    def start_day(self) -> int:
+    def start_day(self) -> int:  # pragma: no cover
         """Returns the EnergyPlus simulation start day of the month.
 
         Returns:
@@ -566,7 +614,7 @@ class EnergyPlus(object):
         return self._eplus_run_st_day
 
     @property
-    def start_weekday(self) -> int:
+    def start_weekday(self) -> int:  # pragma: no cover
         """Returns the EnergyPlus simulation start weekday. From 0 (Monday) to 6 (Sunday).
 
         Returns:
@@ -575,7 +623,7 @@ class EnergyPlus(object):
         return self._eplus_run_st_weekday
 
     @property
-    def env_name(self) -> str:
+    def env_name(self) -> str:  # pragma: no cover
         """Returns the environment name.
 
         Returns:
