@@ -3,7 +3,7 @@
 import os
 from typing import Optional, Union
 
-import gym
+import gymnasium as gym
 import numpy as np
 from stable_baselines3.common.callbacks import BaseCallback, EvalCallback
 from stable_baselines3.common.env_util import is_wrapped
@@ -82,7 +82,7 @@ class LoggerCallback(BaseCallback):
         variables = self.training_env.get_attr('variables')[0]['action']
         action = None
         # sinergym action received inner its own setpoints range
-        action_ = info['action_']
+        action_ = info['action']
         try:
             # network output clipped with gym action space
             action = self.locals['clipped_actions'][-1]
@@ -115,10 +115,10 @@ class LoggerCallback(BaseCallback):
             except KeyError:
                 print('Algorithm reward key in locals dict unknown')
 
-        self.ep_powers.append(info['total_power'])
-        self.ep_term_comfort.append(info['comfort_penalty'])
-        self.ep_term_energy.append(info['total_power_no_units'])
-        if(info['comfort_penalty'] != 0):
+        self.ep_powers.append(info['total_energy'])
+        self.ep_term_comfort.append(info['reward_comfort'])
+        self.ep_term_energy.append(info['reward_energy'])
+        if (info['reward_comfort'] != 0):
             self.num_comfort_violation += 1
         self.ep_timesteps += 1
 
@@ -228,7 +228,7 @@ class LoggerEvalCallback(EvalCallback):
 
             # Reset success rate buffer
             self._is_success_buffer = []
-            #episodes_rewards, episodes_lengths, episodes_powers, episodes_comfort_violations, episodes_comfort_penalties, episodes_power_penalties
+            # episodes_rewards, episodes_lengths, episodes_powers, episodes_comfort_violations, episodes_comfort_penalties, episodes_power_penalties
             episodes_data = evaluate_policy(
                 self.model,
                 self.eval_env,
@@ -313,7 +313,7 @@ class LoggerEvalCallback(EvalCallback):
                     self.model.save(
                         os.path.join(
                             self.best_model_save_path,
-                            self.best_model_save_path.split('/')[-1]))
+                            'model.zip'))
                 self.best_mean_reward = mean_reward
                 # Trigger callback if needed
                 if self.callback is not None:
