@@ -1,51 +1,64 @@
 """Class and utilities for set up extra configuration in experiments with Sinergym (extra params, weather_variability, building model modification and files management)"""
 import os
+import random
 import xml.etree.cElementTree as ElementTree
 from copy import deepcopy
 from shutil import rmtree
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas
 from opyplus import Epm, Idd, WeatherData
+from opyplus.epm.record import Record
 
 from sinergym.utils.common import (get_delta_seconds, get_record_keys,
-                                   prepare_batch_from_records)
-from sinergym.utils.constants import (ACTION_DEFINITION_COMPONENTS,
-                                      CONFIG_KEYS, CWD, PKG_DATA_PATH,
-                                      WEEKDAY_ENCODING, YEAR)
+                                   prepare_batch_from_records, to_idf)
+from sinergym.utils.constants import CWD, PKG_DATA_PATH, WEEKDAY_ENCODING, YEAR
 
 
 class Config(object):
     """Config object to manage extra configuration in Sinergym experiments.
 
         :param _idf_path: IDF path origin for apply extra configuration.
-        :param _weather_path: EPW path origin for apply weather to simulation.
+        :param weather_files: weather available files for each episode
+        :param _weather_path: EPW path origin for apply weather to simulation in current episode.
         :param _ddy_path: DDY path origin for get DesignDays and weather Location
         :param experiment_path: Path for Sinergym experiment output
         :param episode_path: Path for Sinergym specific episode (before first simulator reset this param is None)
-        :param extra_config: Number of episodes directories will be stored in experiment_path
+        :param max_ep_store: Number of episodes directories will be stored in experiment_path
         :param config: Dict config with extra configuration which is required to modify IDF model (may be None)
         :param _idd: IDD opyplus object to set up Epm
         :param building: opyplus Epm object with IDF model
         :param ddy_model: opyplus Epm object with DDY model
         :param weather_data: opyplus WeatherData object with EPW data
+        :param action_definition: Dict with action definition to automatic building model preparation.
     """
 
     def __init__(
             self,
-            idf_path: str,
-            weather_path: str,
+            idf_file: str,
+            weather_files: List[str],
             variables: Dict[str, List[str]],
             env_name: str,
             max_ep_store: int,
+            action_definition: Optional[Dict[str, Any]],
             extra_config: Dict[str, Any]):
 
-        self._idf_path = idf_path
-        self._weather_path = weather_path
+        self.pkg_data_path = PKG_DATA_PATH
+
+        # Transform IDF file name in path
+        self._idf_path = os.path.join(
+            self.pkg_data_path, 'buildings', idf_file)
+
+        # Transform EPW file name in path
+        self.weather_files = weather_files
+
+        # Select one weather randomly (if there are more than one)
+        self._weather_path = os.path.join(
+            self.pkg_data_path, 'weather', random.choice(self.weather_files))
         # RDD file name is deducible using idf name (only change .idf by .rdd)
         self._rdd_path = os.path.join(
-            PKG_DATA_PATH,
+            self.pkg_data_path,
             'variables',
             self._idf_path.split('/')[-1].split('.idf')[0] +
             '.rdd')
@@ -55,7 +68,9 @@ class Config(object):
         self.episode_path = None
         self.max_ep_store = max_ep_store
 
+        # Set config and action definition as config attribute
         self.config = extra_config
+        self.action_definition = action_definition
 
         # Variables XML Tree (empty at the beginning)
         self.variables = variables
@@ -79,9 +94,20 @@ class Config(object):
             self.idf_zone_names.append(idf_zone.name.lower())
         # Extract rdd observation variables names
         data = pandas.read_csv(self._rdd_path, skiprows=1)
-        self.rdd_variables_names = list(map(
+        rdd_variable_names = list(map(
             lambda name: name.split(' [')[0],
             data['Variable Name [Units]'].tolist()))
+        rdd_variable_types = data['Var Type (reported time step)'].tolist()
+        assert len(rdd_variable_names) == len(
+            rdd_variable_types), 'RDD file: Number of variable names and variables types column should be the same.'
+        # self.rdd_variables is a dict with keys as name of the variable and
+        # body as variable type (Zone or HVAC)
+        self.rdd_variables = dict()
+        for i, variable_name in enumerate(rdd_variable_names):
+            self.rdd_variables[variable_name] = rdd_variable_types[i]
+
+        # Extract schedulers available in building model
+        self.schedulers = self.get_schedulers()
 
         # Check observation variables definition
         self._check_observation_variables()
@@ -92,6 +118,18 @@ class Config(object):
     #            IDF, variables and Building model adaptation                      #
     # ---------------------------------------------------------------------------- #
 
+    def update_weather_path(self) -> None:
+        """When this method is called, weather file is changed randomly and IDF is adapted to new one.
+        """
+        self._weather_path = os.path.join(
+            self.pkg_data_path, 'weather', random.choice(self.weather_files))
+        self._ddy_path = self._weather_path.split('.epw')[0] + '.ddy'
+        self.ddy_model = Epm.from_idf(
+            self._ddy_path,
+            idd_or_version=self._idd,
+            check_length=False)
+        self.weather_data = WeatherData.from_epw(self._weather_path)
+
     def adapt_idf_to_epw(self,
                          summerday: str = 'Ann Clg .4% Condns DB=>MWB',
                          winterday: str = 'Ann Htg 99.6% Condns DB') -> None:
@@ -174,6 +212,16 @@ class Config(object):
                 'EnergyPlus',
                 schedule=act_var)
 
+    def set_external_interface(self) -> None:
+        """Set an empty external interface with Ptolemy server if is not in the current building
+        """
+
+        # If no ExternalInterface object found
+        if len(self.building.ExternalInterface) == 0:
+            # Create PtolemyServer interface in building
+            self.building.ExternalInterface.add(
+                name_of_external_interface='PtolemyServer')
+
     def apply_extra_conf(self) -> None:
         """Set extra configuration in building model
         """
@@ -194,51 +242,28 @@ class Config(object):
                 runperiod.end_month = int(self.config['runperiod'][4])
                 runperiod.end_year = int(self.config['runperiod'][5])
 
-            # Action space definition when IDF has not been manipulated
-            # manually
-            if self.config.get('action_definition'):
-                action_definition = self.config['action_definition']
-                for controller_type, controllers in action_definition.items():
-                    # ThermostatSetpoint:DualSetpoint IDF Management
-                    if controller_type == 'ThermostatSetpoint:DualSetpoint':
-                        for controller in controllers:
-                            # Create Ptolomy variables
-                            self.building.ExternalInterface_Schedule.add(
-                                name=controller['heating_name'],
-                                schedule_type_limits_name='Temperature',
-                                initial_value=21)
-                            self.building.ExternalInterface_Schedule.add(
-                                name=controller['cooling_name'],
-                                schedule_type_limits_name='Temperature',
-                                initial_value=25)
-                            # Create a ThermostatSetpoint:DualSetpoint object
-                            self.building.ThermostatSetpoint_DualSetpoint.add(
-                                name=controller['name'],
-                                heating_setpoint_temperature_schedule_name=controller['heating_name'],
-                                cooling_setpoint_temperature_schedule_name=controller['cooling_name'])
-                            # Link in zones required
-                            for zone_control in self.building.ZoneControl_Thermostat:
-                                # If zone specified in zone_control is included
-                                # in out DualSetpoint:
-                                if zone_control.zone_or_zonelist_name.name.lower() in list(
-                                        map(lambda zone: zone.lower(), controller['zones'])):
-                                    # We iterate all record fields searching
-                                    # 'ThermostatSetpoint:DualSetpoint' value
-                                    for i in range(
-                                            len(get_record_keys(zone_control))):
-                                        if isinstance(zone_control[i], str):
-                                            if zone_control[i].lower(
-                                            ) == controller_type.lower():
-                                                # Then, the next field will be
-                                                # always the DualSetpoint
-                                                # thermostat name, so we change
-                                                # it
-                                                zone_control[i +
-                                                             1] = controller['name']
-                                                # We do not need to search more
-                                                # fields in that record
-                                                # specifically, so break it
-                                                break
+    def adapt_idf_to_action_definition(self) -> None:
+        """Interpret action definition and apply changes in IDF in order to control schedulers specified.
+        """
+        if self.action_definition is not None:
+            # Iterate in schedulers to control in action definition
+            for original_sch_name, new_sch in self.action_definition.items():
+                # Search original scheduler information in building model
+                original_sch = self.schedulers[original_sch_name.lower()]
+                # Add external interface to IDF
+                self.building.ExternalInterface_Schedule.add(
+                    name=new_sch['name'],
+                    schedule_type_limits_name=original_sch['Type'],
+                    initial_value=new_sch['initial_value'])
+                # Look for scheduler elements where appear and substitute for new
+                # one name
+                for key, element in original_sch.items():
+                    if isinstance(element, dict):
+                        table_name = element['object_type'].replace(
+                            ':', '_').replace(' ', '_').lower()
+                        record = self.building._tables[table_name].one(
+                            lambda record: record.name == element['object_name'])
+                        record[element['object_field_name']] = new_sch['name']
 
     def save_variables_cfg(self) -> str:
         """This method saves current XML variables tree model into a variables.cfg file.
@@ -277,12 +302,43 @@ class Config(object):
         if self.episode_path is not None:
             episode_idf_path = os.path.join(self.episode_path,
                                             os.path.basename(self._idf_path))
-            self.building.save(episode_idf_path)
+            # self.building.save(episode_idf_path)
+            to_idf(building=self.building, file_path=episode_idf_path)
             return episode_idf_path
         else:
             raise RuntimeError(
                 '[Simulator Config] Episode path should be set before saving building model.')
 
+    def get_schedulers(self) -> Dict[str, Any]:
+        """Extract all schedulers available in the building model to be controlled.
+
+        Returns:
+            Dict[str, Any]: Python Dictionary: For each scheduler found, it shows type value and where this scheduler is present (Object name, Object field and Object type).
+        """
+        result = {}
+        schedule_tables = [
+            self.building.schedule_compact,
+            self.building.schedule_year]
+        for schedule_table in schedule_tables:
+            for schedule in schedule_table:
+                object_index = 1
+                result[schedule.name] = {}
+                result[schedule.name]['Type'] = schedule.schedule_type_limits_name.name
+                # Extract objects where scheduler is present as value field
+                for table in self.building:
+                    if table.get_name() != 'Schedule:Compact' and table.get_name() != 'Schedule:Year':
+                        for record in table:
+                            for i, value in enumerate(record):
+                                if isinstance(value, Record):
+                                    if (value._table._dev_descriptor.table_name == 'Schedule:Compact' or value._table._dev_descriptor.table_name ==
+                                            'Schedule:Year') and value.name == schedule.name:
+                                        result[schedule.name]['Object' + str(object_index)] = {'object_name': record.name,
+                                                                                               'object_field_name': record._table._dev_descriptor._field_descriptors[i].ref,
+                                                                                               'object_type': record._table._dev_descriptor.table_name}
+                                        object_index += 1
+
+        return result
+
     # ---------------------------------------------------------------------------- #
     #                        EPW and Weather Data management                       #
     # ---------------------------------------------------------------------------- #
@@ -300,11 +356,13 @@ class Config(object):
         Returns:
             str: New EPW file path generated in simulator working path in that episode or current EPW path if variation is not defined.
         """
-        if variation is None:
-            return self._weather_path
-        else:
-            # deepcopy for weather_data
-            weather_data_mod = deepcopy(self.weather_data)
+        # deepcopy for weather_data
+        weather_data_mod = deepcopy(self.weather_data)
+        filename = self._weather_path.split('/')[-1]
+
+        # Apply variation to EPW if exists
+        if variation is not None:
+
             # Get dataframe with weather series
             df = weather_data_mod.get_weather_series()
 
@@ -336,13 +394,14 @@ class Config(object):
             # Save new weather data
             weather_data_mod.set_weather_series(df)
 
-            filename = self._weather_path.split('/')[-1]
+            # Change name filename to specify variation nature in name
             filename = filename.split('.epw')[0]
             filename += '_Random_%s_%s_%s.epw' % (
                 str(sigma), str(mu), str(tau))
-            episode_weather_path = self.episode_path + '/' + filename
-            weather_data_mod.to_epw(episode_weather_path)
-            return episode_weather_path
+
+        episode_weather_path = self.episode_path + '/' + filename
+        weather_data_mod.to_epw(episode_weather_path)
+        return episode_weather_path
 
     # ---------------------------------------------------------------------------- #
     #                        Model and Config Functionality                        #
@@ -374,7 +433,7 @@ class Config(object):
         start_weekday = WEEKDAY_ENCODING[runperiod.day_of_week_for_start_day.lower(
         )]
         n_steps_per_hour = self.building.timestep[0].number_of_timesteps_per_hour
-        if n_steps_per_hour < 1 or n_steps_per_hour is None:
+        if n_steps_per_hour < 1 or n_steps_per_hour is None:  # pragma: no cover
             n_steps_per_hour = 4  # default value
 
         return (
@@ -522,8 +581,8 @@ class Config(object):
             rm_dir_full_name = cur_dir_name + base_name + str(rm_dir_id)
             rmtree(rm_dir_full_name)
 
-    @property
-    def start_year(self) -> int:
+    @ property
+    def start_year(self) -> int:  # pragma: no cover
         """Returns the EnergyPlus simulation year.
 
         Returns:
@@ -540,55 +599,101 @@ class Config(object):
         """Check Eplus Environment config definition is correct.
         """
 
-        # Check all keys specified in config are valids (previusly defined in
-        # CONFIG_KEYS)
+        # COMMON
+        # Check weather files exist
+        for w_file in self.weather_files:
+            w_path = os.path.join(
+                self.pkg_data_path, 'weather', w_file)
+            assert os.path.isfile(
+                w_path), 'Weather files: {} is not a weather file available in Sinergym.'.format(w_file)
+
+        # EXTRA CONFIG
         if self.config is not None:
             for config_key in self.config.keys():
-                assert config_key in CONFIG_KEYS, 'Extra parameter {} unknown by Sinergym'.format(
-                    config_key)
-            # Check config parameters values
-            # Timesteps
-            if self.config.get('timesteps_per_hour'):
-                assert self.config['timesteps_per_hour'] > 0, 'timestep_per_hour must be a positive int value.'
-            # Runperiod
-            if self.config.get('runperiod'):
-                assert isinstance(self.config['runperiod'], tuple) and len(
-                    self.config['runperiod']) == 6, 'Runperiod specified in extra configuration has an incorrect format (tuple with 6 elements).'
-            # Action definition
-            if self.config.get('action_definition'):
-                # Check Action definition keys are valids (previusly defined
-                # ACTION_DEFINITION_COMPONENTS)
-                for component_name in self.config['action_definition'].keys():
-                    assert component_name in ACTION_DEFINITION_COMPONENTS, 'The element {} cannot be processed by Sinergym.'.format(
-                        component_name)
-                    # Check ThermostatSetpoint:DualSetpoint
-                    if component_name == 'ThermostatSetpoint:DualSetpoint':
-                        for thermostat in self.config['action_definition'][component_name]:
-                            # Check Thermostates fields
-                            assert set(thermostat.keys()) == set(['name', 'heating_name', 'cooling_name', 'zones']
-                                                                 ), 'Extra config action definition: ThermostatSetpoint:DualSetpoint key names unknown, check them please.'
-                            assert thermostat['heating_name'] in self.variables['action'], 'Extra config action definition: {} should be in action variables.'.format(
-                                thermostat['heating_name'])
-                            assert thermostat['cooling_name'] in self.variables['action'], 'Extra config action definition: {} should be in action variables.'.format(
-                                thermostat['cooling_name'])
-                            for zone in thermostat['zones']:
-                                assert zone.lower() in self.idf_zone_names, 'Extra config action definition: Zone called {} does not exist in IDF building model.'.format(
-                                    zone)
+                # Check config parameters values
+                # Timesteps
+                if config_key == 'timesteps_per_hour':
+                    assert self.config[config_key] > 0, 'Extra Config: timestep_per_hour must be a positive int value.'
+                # Runperiod
+                elif config_key == 'runperiod':
+                    assert isinstance(
+                        self.config[config_key], tuple) and len(
+                        self.config[config_key]) == 6, 'Extra Config: Runperiod specified in extra configuration has an incorrect format (tuple with 6 elements).'
+                else:  # pragma: no cover
+                    raise KeyError(
+                        F'Extra Config: Key name specified in config called [{config_key}] has no support in Sinergym.')
+        # ACTION DEFINITION
+        if self.action_definition is not None:
+            for original_sch_name, new_sch in self.action_definition.items():
+                # Check action definition format
+                assert isinstance(
+                    original_sch_name, str), 'Action definition: Keys must be str.'
+                assert isinstance(
+                    new_sch, dict), 'Action definition: New scheduler definition must be a dict.'
+                assert set(
+                    new_sch.keys()) == set(
+                    ['name', 'initial_value']), 'Action definition: keys in new scheduler definition must be name and initial_value.'
+                assert isinstance(
+                    new_sch['name'], str), 'Action definition: Name field in new scheduler must be a str element.'
+                # Check action definition component is in schedulers available
+                # in building model
+                assert original_sch_name.lower() in self.schedulers.keys(
+                ), 'Action definition: Object called {} is not an existing component in building model.'.format(original_sch_name)
+                # Check new variable is present in action variables
+                assert new_sch['name'] in self.variables['action'], 'Action definition: {} external variable should be in action variables.'.format(
+                    new_sch['name'])
 
     def _check_observation_variables(self) -> None:
-        """This method checks whether observation variables zones are available in building model definition
+        """This method checks whether observation variables are available in building model definition (Checking variable type definition too).
         """
         for obs_var in self.variables['observation']:
+            # Name of the observation variable and element (Zone or HVAC
+            # element name)
             obs_name = obs_var.split('(')[0]
-            obs_zone = obs_var.split('(')[1][:-1]
+            obs_element_name = obs_var.split('(')[1][:-1]
 
             # Check observarion variables names
-            assert obs_name in self.rdd_variables_names, 'Observation variables: Variable called {} in observation variables is not valid for IDF building model'.format(
-                obs_name)
-            # Check observation variables zones
-            if obs_zone.lower() != 'Environment'.lower(
-            ) and obs_zone.lower() != 'Whole Building'.lower():
-                # zones names with people 1 or lights 1, etc. The second name
-                # is ignored, only check that zone is a substr from obs zone
-                assert any(list(map(lambda zone: zone.lower() in obs_zone.lower(), self.idf_zone_names))
-                           ), 'Observation variables: Zone called {} in observation variables does not exist in IDF building model.'.format(obs_zone)
+            assert obs_name in list(self.rdd_variables.keys(
+            )), 'Observation variables: Variable called {} in observation variables is not valid for IDF building model'.format(obs_name)
+            # Check observation variables about zones (if variable type is
+            # Zone)
+            if self.rdd_variables[obs_name] == 'Zone':
+                # Check that obs zone is not Environment or Whole building tag
+                if obs_element_name.lower() != 'Environment'.lower(
+                ) and obs_element_name.lower() != 'Whole Building'.lower():
+                    # zones names with people 1 or lights 1, etc. The second name
+                    # is ignored, only check that zone is a substr from obs
+                    # zone
+                    assert any(list(map(lambda zone: zone.lower() in obs_element_name.lower(), self.idf_zone_names))
+                               ), 'Observation variables: Zone called {} in observation variables does not exist in IDF building model.'.format(obs_element_name)
+            # Check observation variables about HVAC
+            elif self.rdd_variables[obs_name] == 'HVAC':
+                pass
+
+                # ---------------------------------------------------------------------------- #
+                #                                    OTHERS                                    #
+                # ---------------------------------------------------------------------------- #
+
+                # def _set_thermostat_zone_control(self,
+                #                                  zone_control: Record,
+                #                                  controller_type: str,
+                #                                  controller: Dict[str,
+                #                                                   Any]) -> None:
+                #     """Modify Thermostat:ZoneControl object from IDF model in order to set a new controller
+
+                #     Args:
+                #         zone_control (Record): Zone Control to be modified.
+                #         controller_type (str): Controller type you want to change.
+                #         controller (Dict[str, Any]): controller to set in Zone Control.
+                #     """
+                #     # We iterate all record fields searching
+                #     # controller_type value
+                #     for i in range(len(get_record_keys(zone_control))):
+                #         if isinstance(zone_control[i], str):
+                #             if zone_control[i].lower() == controller_type.lower():
+                #                 # Then, the next field will be always the DualSetpoint
+                #                 # thermostat name, so we change it
+                #                 zone_control[i + 1] = controller['name']
+                #                 # We do not need to search more fields in that record
+                #                 # specifically, so break it
+                #                 break
