@@ -5,7 +5,7 @@ from datetime import datetime
 from math import exp
 from typing import Any, Dict, List, Tuple, Union
 
-from gym import Env
+from gymnasium import Env
 
 
 class BaseReward(object):
@@ -20,7 +20,6 @@ class BaseReward(object):
             env (Env): Gym environment.
         """
         self.env = env
-        self.year = 2021  # just for datetime completion
 
     def __call__(self):
         """Method for calculating the reward function."""
@@ -35,8 +34,10 @@ class LinearReward(BaseReward):
         env: Env,
         temperature_variable: Union[str, list],
         energy_variable: str,
-        range_comfort_winter: tuple,
-        range_comfort_summer: tuple,
+        range_comfort_winter: Tuple[int, int],
+        range_comfort_summer: Tuple[int, int],
+        summer_start: Tuple[int, int] = (6, 1),
+        summer_final: Tuple[int, int] = (9, 30),
         energy_weight: float = 0.5,
         lambda_energy: float = 1e-4,
         lambda_temperature: float = 1.0
@@ -53,8 +54,10 @@ class LinearReward(BaseReward):
             env (Env): Gym environment.
             temperature_variable (Union[str, list]): Name(s) of the temperature variable(s).
             energy_variable (str): Name of the energy/power variable.
-            range_comfort_winter (tuple): Temperature comfort range for cold season. Depends on environment you are using.
-            range_comfort_summer (tuple): Temperature comfort range fot hot season. Depends on environment you are using.
+            range_comfort_winter (Tuple[int,int]): Temperature comfort range for cold season. Depends on environment you are using.
+            range_comfort_summer (Tuple[int,int]): Temperature comfort range for hot season. Depends on environment you are using.
+            summer_start (Tuple[int,int]): Summer session tuple with month and day start. Defaults to (6,1).
+            summer_final (Tuple[int,int]): Summer session tuple with month and day end. defaults to (9,30).
             energy_weight (float, optional): Weight given to the energy term. Defaults to 0.5.
             lambda_energy (float, optional): Constant for removing dimensions from power(1/W). Defaults to 1e-4.
             lambda_temperature (float, optional): Constant for removing dimensions from temperature(1/C). Defaults to 1.0.
@@ -73,9 +76,9 @@ class LinearReward(BaseReward):
         self.lambda_energy = lambda_energy
         self.lambda_temp = lambda_temperature
 
-        # Periods
-        self.summer_start_date = datetime(self.year, 6, 1)
-        self.summer_final_date = datetime(self.year, 9, 30)
+        # Summer period
+        self.summer_start = summer_start  # (month,day)
+        self.summer_final = summer_final  # (month,day)
 
     def __call__(self) -> Tuple[float, Dict[str, Any]]:
         """
@@ -100,8 +103,9 @@ class LinearReward(BaseReward):
 
         reward_terms = {
             'reward_energy': reward_energy,
-            'total_energy': obs_dict[self.energy_name],
             'reward_comfort': reward_comfort,
+            'total_energy': obs_dict[self.energy_name],
+            'abs_comfort': comfort,
             'temperatures': temps
         }
 
@@ -119,9 +123,20 @@ class LinearReward(BaseReward):
 
         month = obs_dict['month']
         day = obs_dict['day']
-        current_dt = datetime(self.year, month, day)
+        year = obs_dict['year']
+        current_dt = datetime(year, month, day)
 
-        if current_dt >= self.summer_start_date and current_dt <= self.summer_final_date:
+        # Periods
+        summer_start_date = datetime(
+            year,
+            self.summer_start[0],
+            self.summer_start[1])
+        summer_final_date = datetime(
+            year,
+            self.summer_final[0],
+            self.summer_final[1])
+
+        if current_dt >= summer_start_date and current_dt <= summer_final_date:
             temp_range = self.range_comfort_summer
         else:
             temp_range = self.range_comfort_winter
@@ -142,8 +157,10 @@ class ExpReward(LinearReward):
         env: Env,
         temperature_variable: Union[str, list],
         energy_variable: str,
-        range_comfort_winter: tuple,
-        range_comfort_summer: tuple,
+        range_comfort_winter: Tuple[int, int],
+        range_comfort_summer: Tuple[int, int],
+        summer_start: Tuple[int, int] = (6, 1),
+        summer_final: Tuple[int, int] = (9, 30),
         energy_weight: float = 0.5,
         lambda_energy: float = 1e-4,
         lambda_temperature: float = 1.0
@@ -158,8 +175,10 @@ class ExpReward(LinearReward):
             env (Env): Gym environment.
             temperature_variable (Union[str, list]): Name(s) of the temperature variable(s).
             energy_variable (str): Name of the energy/power variable.
-            range_comfort_winter (tuple): Temperature comfort range for cold season. Depends on environment you are using.
-            range_comfort_summer (tuple): Temperature comfort range fot hot season. Depends on environment you are using.
+            range_comfort_winter (Tuple[int,int]): Temperature comfort range for cold season. Depends on environment you are using.
+            range_comfort_summer (Tuple[int,int]): Temperature comfort range for hot season. Depends on environment you are using.
+            summer_start (Tuple[int,int]): Summer session tuple with month and day start. Defaults to (6,1).
+            summer_final (Tuple[int,int]): Summer session tuple with month and day end. defaults to (9,30).
             energy_weight (float, optional): Weight given to the energy term. Defaults to 0.5.
             lambda_energy (float, optional): Constant for removing dimensions from power(1/W). Defaults to 1e-4.
             lambda_temperature (float, optional): Constant for removing dimensions from temperature(1/C). Defaults to 1.0.
@@ -171,6 +190,8 @@ class ExpReward(LinearReward):
             energy_variable,
             range_comfort_winter,
             range_comfort_summer,
+            summer_start,
+            summer_final,
             energy_weight,
             lambda_energy,
             lambda_temperature
@@ -188,9 +209,20 @@ class ExpReward(LinearReward):
 
         month = obs_dict['month']
         day = obs_dict['day']
-        current_dt = datetime(self.year, month, day)
+        year = obs_dict['year']
+        current_dt = datetime(year, month, day)
 
-        if current_dt >= self.summer_start_date and current_dt <= self.summer_final_date:
+        # Periods
+        summer_start_date = datetime(
+            year,
+            self.summer_start[0],
+            self.summer_start[1])
+        summer_final_date = datetime(
+            year,
+            self.summer_final[0],
+            self.summer_final[1])
+
+        if current_dt >= summer_start_date and current_dt <= summer_final_date:
             temp_range = self.range_comfort_summer
         else:
             temp_range = self.range_comfort_winter
@@ -212,8 +244,10 @@ class HourlyLinearReward(LinearReward):
         env: Env,
         temperature_variable: Union[str, list],
         energy_variable: str,
-        range_comfort_winter: tuple,
-        range_comfort_summer: tuple,
+        range_comfort_winter: Tuple[int, int],
+        range_comfort_summer: Tuple[int, int],
+        summer_start: Tuple[int, int] = (6, 1),
+        summer_final: Tuple[int, int] = (9, 30),
         min_energy_weight: float = 0.5,
         lambda_energy: float = 1e-4,
         lambda_temperature: float = 1.0,
@@ -226,8 +260,10 @@ class HourlyLinearReward(LinearReward):
             env (Env): Gym environment.
             temperature_variable (Union[str, list]): Name(s) of the temperature variable(s).
             energy_variable (str): Name of the energy/power variable.
-            range_comfort_winter (tuple): Temperature comfort range for cold season. Depends on environment you are using.
-            range_comfort_summer (tuple): Temperature comfort range fot hot season. Depends on environment you are using.
+            range_comfort_winter (Tuple[int,int]): Temperature comfort range for cold season. Depends on environment you are using.
+            range_comfort_summer (Tuple[int,int]): Temperature comfort range for hot season. Depends on environment you are using.
+            summer_start (Tuple[int,int]): Summer session tuple with month and day start. Defaults to (6,1).
+            summer_final (Tuple[int,int]): Summer session tuple with month and day end. defaults to (9,30).
             min_energy_weight (float, optional): Minimum weight given to the energy term. Defaults to 0.5.
             lambda_energy (float, optional): Constant for removing dimensions from power(1/W). Defaults to 1e-4.
             lambda_temperature (float, optional): Constant for removing dimensions from temperature(1/C). Defaults to 1.0.
@@ -240,6 +276,8 @@ class HourlyLinearReward(LinearReward):
             energy_variable,
             range_comfort_winter,
             range_comfort_summer,
+            summer_start,
+            summer_final,
             min_energy_weight,
             lambda_energy,
             lambda_temperature
@@ -276,8 +314,9 @@ class HourlyLinearReward(LinearReward):
 
         reward_terms = {
             'reward_energy': reward_energy,
-            'total_energy': obs_dict[self.energy_name],
             'reward_comfort': reward_comfort,
+            'total_energy': obs_dict[self.energy_name],
+            'abs_comfort': comfort,
             'temperatures': temps
         }
 
